@@ -2,10 +2,14 @@ import pandas as pd
 import numpy as np
 import gwaslab as gl
 import time
+from gwaslab.Sumstatssummary import summarize
+from gwaslab.Sumstatssummary import lookupstatus
+
 #20220309
 class Sumstats():
     def __init__(self,
              sumstats,
+             fmt=None,
              snpid=None,
              rsid=None,
              chrom=None,
@@ -13,6 +17,7 @@ class Sumstats():
              ea=None,
              nea=None,
              eaf=None,
+             neaf=None,
              n=None,
              beta=None,
              se=None,
@@ -38,7 +43,9 @@ class Sumstats():
         self.log = gl.Log()
         
         #preformat the data
-        self.data  = gl.preformat(sumstats=sumstats,
+        self.data  = gl.preformat(
+          sumstats=sumstats,
+          fmt=fmt,
           snpid=snpid,
           rsid=rsid,
           chrom=chrom,
@@ -46,6 +53,7 @@ class Sumstats():
           ea=ea,
           nea=nea,
           eaf=eaf,
+          neaf=neaf,
           n=n,
           beta=beta,
           se=se,
@@ -99,14 +107,18 @@ class Sumstats():
                 col_subset[dic[key]] = key
         if len(other)>0: col_subset["other"] = other      
         return col_subset
-    
+
+##########################################################################################
     def update_meta(self):
         self.meta["Number_of_variants"]=len(self.data)
         if "CHR" in self.data.columns:
             self.meta["Number_of_chromosomes"]=len(self.data["CHR"].unique())
         if "P" in self.data.columns:
             self.meta["Min_P"]=np.min(self.data["P"])
-        
+    def summary(self):
+        return summarize(self.data)
+    def lookup_status(self,status="STATUS"):
+        return lookupstatus(self.data[status])
         
 # QC ######################################################################################
     #clean the sumstats with one line
@@ -134,21 +146,14 @@ class Sumstats():
         ###############################################
         
     
-    def clean(self,
+    def harmonize(self,
               basic_check=True,
               ref_seq=None,
-              ref_rsid=None,
-              ref_rsid_chr_dict=None,
+              ref_rsid_tsv=None,
+              ref_rsid_vcf=None,
               ref_infer=None,
-              ref_infer_chr_dict=None,
               ref_alt_freq=None,
-              from_build=None,
-              to_build=None,
-              to_ref_seq=None,
-              to_ref_infer=None,
-              to_ref_rsid=None,
-              to_ref_alt_freq=None,
-              maf_threshold=0.43,
+              maf_threshold=0.40,
               n_cores=1,
               remove=False,
               checkref_args={},
@@ -175,12 +180,19 @@ class Sumstats():
         #    1.5 sanity check : BETA SE OR EAF N OR_95L OR_95H
         #    1.6 sorting genomic coordinates and column order 
         if basic_check is True:
+            
             self.data = gl.fixID(self.data,**fixid_args)
+            
             self.data = gl.fixchr(self.data,remove=remove,log=self.log,**fixchr_agrs)
+            
             self.data = gl.fixpos(self.data,remove=remove,log=self.log,**fixpos_args)
+            
             self.data = gl.fixallele(self.data,log=self.log,**fixallele_args)
+            
             self.data = gl.sanitycheckstats(self.data,log=self.log,**sanitycheckstats_args)
-            self.data = gl.parallelnormalizeallele(self.data,log=self.log,**normalizeallele_args)
+            
+            self.data = gl.parallelnormalizeallele(self.data,log=self.log,n_cores=n_cores,**normalizeallele_args)
+            
             self.data = gl.sortcolumn(self.data,log=self.log)
         
         
@@ -198,38 +210,41 @@ class Sumstats():
         #   3.2 infer strand for palindromic SNP (target build)
         #####################################################
         if ref_seq is not None:
+            
             self.data = gl.checkref(self.data,ref_seq,log=self.log,**checkref_args)
+            
             self.data = gl.flipallelestats(self.data,log=self.log,**flipallelestats_args)
-        
-        if ref_rsid is not None:
-            self.data = gl.parallelizeassignrsid(self.data,ref_rsid,n_cores=n_cores,log=self.log,chr_dict=ref_rsid_chr_dict,**assignrsid_args)
             
         if ref_infer is not None: 
-            self.data= gl.inferstrand(self.data,ref_infer = ref_infer,ref_alt_freq=ref_alt_freq,maf_threshold=0.43,chr_dict=ref_infer_chr_dict,log=self.log,**inferstrand_args)
-            self.data =gl.flipallelestats(self.data,log=self.log,**flipallelestats_args)
             
-        #if liftover           
-        if to_build is not None :
-            self.data = gl.parallelizeliftovervariant(self.data,n_cores=n_cores,from_build=from_build,to_build=to_build,log=self.log,**liftover_args)
-            if to_ref_seq is not None:
-                self.data = gl.checkref(self.data,to_ref_seq,log=self.log,**checkref_args)
-                self.data = gl.flipallelestats(self.data,log=self.log,**flipallelestats_args)
-                if to_ref_rsid is not None  :
-                    self.data = gl.parallelizeassignrsid(self.data,to_ref_rsid,n_cores=n_cores,log=self.log,**assignrsid_args)
-                if (to_ref_infer is not None) and (to_ref_alt_freq is not None): 
-                    self.data = gl.inferstrand(self.data,ref_infer = to_ref_infer,ref_alt_freq=to_ref_alt_freq,maf_threshold=0.43,log=self.log,**inferstrand_args)
-                    self.data = gl.flipallelestats(self.data,log=self.log,**flipallelestats_args)
-        self.data = gl.fixchr(self.data, remove=remove)  
+            self.data= gl.parallelinferstrand(self.data,ref_infer = ref_infer,ref_alt_freq=ref_alt_freq,
+                                              n_cores=n_cores,log=self.log,**inferstrand_args)
+            
+            self.data =gl.flipallelestats(self.data,log=self.log,**flipallelestats_args)
+        
+        #####################################################
+        if ref_rsid_tsv is not None:
+            
+            self.data = gl.parallelizeassignrsid(self.data,path=ref_rsid_tsv,ref_mode="tsv",
+                                                 n_cores=n_cores,log=self.log,**assignrsid_args)
+        if ref_rsid_vcf is not None:
+            
+            self.data = gl.parallelizeassignrsid(self.data,path=ref_rsid_vcf,ref_mode="vcf",
+                                                 n_cores=n_cores,log=self.log,**assignrsid_args)     
+        ######################################################    
         if remove is True:
+            
             self.data = gl.removedup(self.data,log=self.log,**removedup_args)
         ################################################ 
+        
         self.data = gl.sortcoordinate(self.data,log=self.log)
+        
         self.data = gl.sortcolumn(self.data,log=self.log)
+        
         return self
-    
     ############################################################################################################
     #customizable API to build your own QC pipeline
-    def fix_ID(self,**args):
+    def fix_id(self,**args):
         self.data = gl.fixID(self.data,log=self.log,**args)
     def fix_chr(self,**args):
         self.data = gl.fixchr(self.data,log=self.log,**args)
@@ -241,7 +256,7 @@ class Sumstats():
         self.data = gl.removedup(self.data,log=self.log,**args)
     def check_sanity(self,**args):
         self.data = gl.sanitycheckstats(self.data,log=self.log,**args)
-    def check_ID(self,**args):
+    def check_id(self,**args):
         pass
     def check_ref(self,**args):
         self.data = gl.checkref(self.data,log=self.log,**args)
@@ -251,8 +266,15 @@ class Sumstats():
         self.data = gl.flipallelestats(self.data,log=self.log,**args)
     def normalize_allele(self,**args):
         self.data = gl.parallelnormalizeallele(self.data,log=self.log,**args)
-    def assign_rsid(self,**args):
-        self.data = gl.parallelizeassignrsid(self.data,log=self.log,**args)
+    def assign_rsid(self,
+                    ref_rsid_tsv=None,
+                    ref_rsid_vcf=None,
+                    **args):
+        if ref_rsid_tsv is not None:
+            self.data = gl.parallelizeassignrsid(self.data,path=ref_rsid_tsv,ref_mode="tsv",log=self.log,**args)
+        if ref_rsid_vcf is not None:
+            self.data = gl.parallelizeassignrsid(self.data,path=ref_rsid_vcf,ref_mode="vcf",log=self.log,**args)   
+        
     def rsid_to_chrpos(self,**args):
         self.data = gl.rsidtochrpos(self.data,log=self.log,**args)
     def rsid_to_chrpos2(self,**args):
@@ -279,9 +301,20 @@ class Sumstats():
             return gl.filterin(self.data,**args)
         else:
             self.data = gl.filterin(self.data,log=self.log,**args)
+    def filter_region_in(self, inplace=True, **args):
+        if inplace is False:
+            return gl.filterregionin(self.data,**args)
+        else:
+            self.data = gl.filterregionin(self.data,log=self.log,**args)
+    def filter_region_out(self, inplace=True, **args):
+        if inplace is False:
+            return gl.filterregionout(self.data,**args)
+        else:
+            self.data = gl.filterregionout(self.data,log=self.log,**args)        
+    
 ###########################################################################################################################            
     def check_af(self,**args):
-        self.data = gl.checkaf(self.data,log=self.log,**args)
+        self.data = gl.parallelecheckaf(self.data,log=self.log,**args)
       
     def plot_daf(self, **args):
         plot = gl.plotdaf(self.data, **args)
@@ -348,21 +381,24 @@ class Sumstats():
 # to_format ###############################################################################################       
     def to_format(self,
               path="./sumstats",
-              format="ldsc",   
+              fmt="ldsc",   
               extract=None,
               exclude=None,
               id_use="rsID",
               hapmap3=False,
-              exclude_hla=False,        
+              exclude_hla=False,  
+              build="19", 
               verbose=True,
               output_log=True,
-              to_csvargs={}):
+              to_csvargs={},
+              float_formats={}):
         
-        if format in ["ldsc","fuma","metal","bed","vcf"]:
-            if verbose: self.log.write("Output sumstats to format:",format)
+        if fmt in ["ldsc","fuma","metal","bed","vcf","fastgwa","ssf","plink","plink2","saige","regenie","gwascatalog","pgscatalog"]:
+            if verbose: self.log.write("Output sumstats to format:",fmt)
         else:
             raise ValueError("Please select a format to output")
         
+        suffix=fmt
         #######################################################################################################
         # filter
         output = self.data.copy()
@@ -374,25 +410,64 @@ class Sumstats():
 
         if exclude_hla is True:
             if verbose: self.log.write(" -Excluding variants in HLA region ...")
-            is_hla = (output["CHR"]== "6") & (output["POS"] >25000000) & (output["POS"] < 34000000)
+            before = len(output)
+            is_hla = (output["CHR"]== 6) & (output["POS"].astype("Int64") >25000000) & (output["POS"].astype("Int64") < 34000000)
             output = output.loc[~is_hla,:]
-
+            after = len(output)
+            if verbose: self.log.write(" -Exclude "+ str(before - after) + " variants in HLA region.")
+            suffix = "noMHC."+suffix
+        
         if hapmap3 is True:
-            output = gl.gethapmap3(output)
+            output = gl.gethapmap3(output,build=build,verbose=True)
+            after = len(output)
+            if verbose: self.log.write(" -Extract "+ str(after) + " variants in Hapmap3 datasets for build "+build+".")
+            suffix = "hapmap3."+suffix
+                
         #######################################################################################################
-
-        # output
-        if format=="ldsc":
-            gl.toldsc(output, path=path+"."+format,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        if format=="fuma":
-            gl.toldsc(output, path=path+"."+format,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        if format=="bed":
-            gl.toldsc(output, path=path+"."+format,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        if format=="metal":
-            gl.toldsc(output, path=path+"."+format,verbose=True,log=self.log,to_csvargs=to_csvargs)
-        if format=="vcf":
-            gl.toldsc(output, path=path+"."+format,verbose=True,log=self.log,to_csvargs=to_csvargs)
+        #formatting statistics
+        if verbose: self.log.write(" -Formatting statistics ...")
+        formats = {'EAF': '{:.4g}', 
+                   'BETA': '{:.4f}', 
+                   'Z': '{:.4f}',
+                   'CHISQ': '{:.4f}',
+                   'SE': '{:.4f}',
+                   'OR': '{:.4f}',
+                   'OR_95U': '{:.4f}',
+                   'OR_95L': '{:.4f}',
+                   'INFO': '{:.4f}',
+                   'P': '{:.4e}',
+                   'MLOG10P': '{:.4f}',
+                   'DAF': '{:.4f}'
+                  }
+        for col, f in float_formats.items():
+            if col in output.columns: 
+                formats[col]=f
+        for col, f in formats.items():
+            if col in output.columns: 
+                if output[col].dtype in ["float64","float32","float16","float"]:
+                    output[col] = output[col].map(f.format)
+        if verbose: 
+            self.log.write(" -Float statistics formats:")  
+            keys=[]
+            values=[]
+            for key,value in formats.items():
+                if key in output.columns: 
+                    keys.append(key)
+                    values.append(value)
+            self.log.write("  - Columns:",keys) 
+            self.log.write("  - Output formats:",values) 
+        ##########################################################################################################          
+        # output, mapping column names
+        if fmt=="ldsc":
+            gl.toldsc(output, path=path+"."+suffix,verbose=True,log=self.log,to_csvargs=to_csvargs)
+        #if fmt=="fuma":
+            #gl.toldsc(output, path=path+"."+fmt,verbose=True,log=self.log,to_csvargs=to_csvargs)
+        #if fmt=="bed":
+            #gl.toldsc(output, path=path+"."+fmt,verbose=True,log=self.log,to_csvargs=to_csvargs)
+        if fmt=="vcf":
+            gl.toldsc(output, path=path+"."+suffix,verbose=True,log=self.log,to_csvargs=to_csvargs)
+        if fmt in ["fastgwa","ssf","plink","plink2","saige","regenie","gwascatalog","pgscatalog"]:
+            gl.to_formats.tofmt(output, path=path,fmt=fmt,suffix=suffix,verbose=True,log=self.log,to_csvargs=to_csvargs)
         if output_log is True:
             if verbose: self.log.write("Saving log file...")
-            self.log.save(path + "."+ format +".log")
-            
+            self.log.save(path + "."+ suffix +".log")
