@@ -51,20 +51,22 @@ def compare_effect(path1,
                    helper_line_args={"color":'black', "linestyle":'-',"lw":1},
                    fontargs={'family':'sans','fontname':'Arial','fontsize':12},
                    errargs={"ecolor":"#cccccc","elinewidth":1},
+                   sep=["\t","\t"],
                    verbose=False):
     
     #[snpid,p,ea,nea]      ,[effect,se]
     #[snpid,p,ea,nea,chr,pos],[effect,se]
     #[snpid,p,ea,nea,chr,pos],[OR,OR_l,OR_h]
+    
     if verbose: print("Start to process the raw sumstats for plotting...")
+    
     ######### 1 check the value used to plot
     if mode not in ["Beta","beta","BETA","OR","or"]:
         raise ValueError("Please input Beta or OR")
     
     ######### 2 extract snplist2
-    if verbose: print(" -Loading "+label[1]+" SNP list in memory...")
-        
-    sumstats=pd.read_table(path2,sep="\s+",usecols=[cols_name_list_2[0]])
+    if verbose: print(" -Loading "+label[1]+" SNP list in memory...")    
+    sumstats=pd.read_table(path2,sep=sep[1],usecols=[cols_name_list_2[0]])
     common_snp_set=set(sumstats[cols_name_list_2[0]].values)
     
     ######### 3 extract snplist1
@@ -75,7 +77,7 @@ def compare_effect(path1,
  
     ######### 4 load sumstats1
     if verbose: print(" -Loading sumstats for "+label[0]+":",",".join(cols_to_extract))
-    sumstats = pd.read_table(path1,sep="\s+",usecols=cols_to_extract)
+    sumstats = pd.read_table(path1,sep=sep[0],usecols=cols_to_extract)
     
     ######### 5 extract the common set
     common_snp_set = common_snp_set.intersection(sumstats[cols_name_list_1[0]].values)
@@ -84,13 +86,15 @@ def compare_effect(path1,
     ######### 6 rename the sumstats
     rename_dict = { cols_name_list_1[0]:"SNPID",
                cols_name_list_1[1]:"P",
-    }
+               }
+    
     if snplist is None: 
         rename_dict[cols_name_list_1[4]]="CHR"
         rename_dict[cols_name_list_1[5]]="POS"
+    
     sumstats.rename(columns=rename_dict,inplace=True)
     
-    ######### 7 exctract only common variants from sumstats1 
+    ######### 7 exctract only available variants from sumstats1 
     sumstats=sumstats.loc[sumstats["SNPID"].isin(common_snp_set),:]
     if verbose: print(" -Using only variants available for both datasets...")
     
@@ -103,8 +107,7 @@ def compare_effect(path1,
     else:
         ######### 8,2 otherwise use the sutomatically detected lead SNPs
         if verbose: print(" -Extract lead variants from "+label[0]+"...")
-        sig_list_1 = gl.getsig(sumstats,"SNPID","CHR","POS","P",
-                               verbose=verbose,sig_level=sig_level)
+        sig_list_1 = gl.getsig(sumstats,"SNPID","CHR","POS","P", verbose=verbose,sig_level=sig_level)
 
     ######### 9 extract snplist2
     if snplist is not None:
@@ -113,12 +116,12 @@ def compare_effect(path1,
         cols_to_extract = [cols_name_list_2[0],cols_name_list_2[1],cols_name_list_2[4],cols_name_list_2[5]]
     
     if verbose: print(" -Loading sumstats for "+label[1]+":",",".join(cols_to_extract))
-    sumstats = pd.read_table(path2,sep="\s+",usecols=cols_to_extract)
+    sumstats = pd.read_table(path2,sep=sep[1],usecols=cols_to_extract)
     
     ######### 10 rename sumstats2
     rename_dict = { cols_name_list_2[0]:"SNPID",
                     cols_name_list_2[1]:"P",
-    }
+                }
     if snplist is None: 
         rename_dict[cols_name_list_2[4]]="CHR"
         rename_dict[cols_name_list_2[5]]="POS"
@@ -148,12 +151,15 @@ def compare_effect(path1,
     ########## 14 Merging sumstats1
     
     if mode=="beta" or mode=="BETA" or mode=="Beta":
+         #[snpid,p,ea,nea]      ,[effect,se]
+        #[snpid,p,ea,nea,chr,pos],[effect,se]
+        #[snpid,p,ea,nea,chr,pos],[OR,OR_l,OR_h]
         cols_to_extract = [cols_name_list_1[0],cols_name_list_1[2],cols_name_list_1[3], effect_cols_list_1[0], effect_cols_list_1[1]]
     else:
         cols_to_extract = [cols_name_list_1[0],cols_name_list_1[2],cols_name_list_1[3], effect_cols_list_1[0], effect_cols_list_1[1], effect_cols_list_1[2]]
     if len(eaf)>0: cols_to_extract.append(eaf[0])   
     if verbose: print(" -Extract statistics of selected variants from "+label[0]+" : ",",".join(cols_to_extract) )
-    sumstats = pd.read_table(path1,sep="\s+",usecols=cols_to_extract)
+    sumstats = pd.read_table(path1,sep=sep[0],usecols=cols_to_extract)
     
     if mode=="beta" or mode=="BETA" or mode=="Beta":
         rename_dict = { cols_name_list_1[0]:"SNPID",
@@ -164,6 +170,7 @@ def compare_effect(path1,
     }
         
     else:
+        # if or
         rename_dict = { cols_name_list_1[0]:"SNPID",
                         cols_name_list_1[2]:"EA_1",
                         cols_name_list_1[3]:"NEA_1",
@@ -171,6 +178,7 @@ def compare_effect(path1,
                         effect_cols_list_1[1]:"OR_L_1",
                         effect_cols_list_1[2]:"OR_H_1"
     }
+    ## check if eaf column is provided.
     if len(eaf)>0: rename_dict[eaf[0]]="EAF_1"
     sumstats.rename(columns=rename_dict, inplace=True)
     
@@ -184,10 +192,12 @@ def compare_effect(path1,
     if mode=="beta" or mode=="BETA" or mode=="Beta":
         cols_to_extract = [cols_name_list_2[0],cols_name_list_2[2],cols_name_list_2[3], effect_cols_list_2[0], effect_cols_list_2[1]]
     else:
+        # if or
         cols_to_extract = [cols_name_list_2[0],cols_name_list_2[2],cols_name_list_2[3], effect_cols_list_2[0], effect_cols_list_2[1], effect_cols_list_2[2]]
+    ## check if eaf column is provided.
     if len(eaf)>0: cols_to_extract.append(eaf[1])
     if verbose: print(" -Extract statistics of selected variants from "+label[1]+" : ",",".join(cols_to_extract) )
-    sumstats = pd.read_table(path2,sep="\s+",usecols=cols_to_extract)
+    sumstats = pd.read_table(path2,sep=sep[1],usecols=cols_to_extract)
     
     if mode=="beta" or mode=="BETA" or mode=="Beta":
           rename_dict = { cols_name_list_2[0]:"SNPID",
@@ -217,7 +227,7 @@ def compare_effect(path1,
 
     ################ 16 update sumstats1
     if verbose: print(" -Updating missing information for "+label[0]+" ...")
-    sumstats = pd.read_table(path1,sep="\s+",usecols=[cols_name_list_1[0],cols_name_list_1[1]])
+    sumstats = pd.read_table(path1,sep=sep[0],usecols=[cols_name_list_1[0],cols_name_list_1[1]])
     sumstats.rename(columns={
                         cols_name_list_1[0]:"SNPID",
                         cols_name_list_1[1]:"P_1"
@@ -228,7 +238,7 @@ def compare_effect(path1,
     
     ################# 17 update sumstats2
     if verbose: print(" -Updating missing information for "+label[1]+" ...")
-    sumstats = pd.read_table(path2,sep="\s+",usecols=[cols_name_list_2[0],cols_name_list_2[1]])
+    sumstats = pd.read_table(path2,sep=sep[1],usecols=[cols_name_list_2[0],cols_name_list_2[1]])
     sumstats.rename(columns={
                         cols_name_list_2[0]:"SNPID",
                         cols_name_list_2[1]:"P_2"
@@ -241,9 +251,14 @@ def compare_effect(path1,
 #################################################################################
     ############## 18 init indicator
     if verbose: print(" -Assigning indicator  ...")
+    # 0-> 0
+    # 1 -> sumstats1
+    # 2 -> sumsatts2
+    # 3->  sumstats3
     sig_list_merged["indicator"] = 0
     sig_list_merged.loc[sig_list_merged["P_1"]<sig_level,"indicator"]=1+sig_list_merged.loc[sig_list_merged["P_1"]<sig_level,"indicator"]
     sig_list_merged.loc[sig_list_merged["P_2"]<sig_level,"indicator"]=2+sig_list_merged.loc[sig_list_merged["P_2"]<sig_level,"indicator"]
+    
     if snplist is None:
         sig_list_merged["CHR"]=np.max(sig_list_merged[["CHR_1","CHR_2"]], axis=1).astype(int)
         sig_list_merged["POS"]=np.max(sig_list_merged[["POS_1","POS_2"]], axis=1).astype(int)
@@ -256,11 +271,12 @@ def compare_effect(path1,
         sig_list_merged["NEA_2_aligned"]=sig_list_merged["NEA_2"]
         sig_list_merged["EFFECT_2_aligned"]=sig_list_merged["EFFECT_2"]
         
+        #filp ea/nea and beta for sumstats2
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"EA_2_aligned"]= sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"NEA_2"]
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"NEA_2_aligned"]= sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"EA_2"]
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"EFFECT_2_aligned"]= -sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"EFFECT_2"]
     else:
-        #or - +
+        #flip for OR or - +
         sig_list_merged["OR_L_1"]=np.abs(sig_list_merged["OR_L_1"]-sig_list_merged["OR_1"])
         sig_list_merged["OR_H_1"]=np.abs(sig_list_merged["OR_H_1"]-sig_list_merged["OR_1"])
         sig_list_merged["OR_L_2"]=np.abs(sig_list_merged["OR_L_2"]-sig_list_merged["OR_2"])
@@ -277,6 +293,8 @@ def compare_effect(path1,
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"OR_2_aligned"]= 1/sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"OR_2"]
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"OR_L_2_aligned"]= 1/sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"OR_L_2"]
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"OR_H_2_aligned"]= 1/sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"OR_H_2"]
+    
+    # flip eaf
     if len(eaf)>0:
         sig_list_merged["EAF_2_aligned"]=sig_list_merged["EAF_2"]
         sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"EAF_2_aligned"]= 1 -sig_list_merged.loc[sig_list_merged["EA_1"]!=sig_list_merged["EA_2"],"EAF_2"]
@@ -361,7 +379,7 @@ def compare_effect(path1,
                         linewidth=0,zorder=1,**errargs)
             ax.scatter(both["OR_1"],both["OR_2_aligned"],label=label[2],zorder=2,color="#e6320e",marker="s",**scatterargs)
     
-    ## annotation
+    ## annotation #################################################################################################################
     if anno==True:
         from adjustText import adjust_text
         sig_list_toanno = sig_list_merged.dropna(axis=0)
@@ -380,6 +398,7 @@ def compare_effect(path1,
         for index, row in sig_list_toanno.iterrows():
             texts.append(plt.text(row["EFFECT_1"], row["EFFECT_2_aligned"],anno[index], ha='right', va='top')) 
         adjust_text(texts,ha='right', va='top',arrowprops=dict(arrowstyle='->', color='grey'))
+    #################################################################################################################################
     
     # plot x=0,y=0, and a 45 degree line
     xl,xh=ax.get_xlim()
@@ -467,8 +486,8 @@ def compare_effect(path1,
 def plotdaf(sumstats,
              eaf="EAF",
              daf="DAF",
-             scatter_args={"s":4},
-             threshold=0.2,
+             scatter_args={"s":1},
+             threshold=0.16,
              is_reg=True,
              is_45_helper_line=True,
              is_threshold=True,
@@ -476,18 +495,23 @@ def plotdaf(sumstats,
              threshold_line_args={"color":'#cccccc', "linestyle":'dotted'},
              reg_line_args={"color":'#cccccc', "linestyle":'--'},
              plt_args={"figsize":(8,4),"dpi":300},
+            histplot_args={"log_scale":(False,True)},
             fontargs={'family':'sans','fontname':'Arial','fontsize':8},
             verbose=True
            ):
     
     
     if verbose: print("Start to plot Reference frequency vs Effect allele frequency plot...")
+    if not ((eaf in sumstats.columns) and (daf in sumstats.columns)):
+        raise ValueError("EAF and/or DAF columns were not detected.")
     
-    sumstats = sumstats.loc[(~sumstats[eaf].isna())&(~sumstats[daf].isna()),:].copy()
+    
+    sumstats = sumstats.loc[(~sumstats[eaf].isna())&(~sumstats[daf].isna()),[eaf,daf]].copy()
+    sumstats.loc[:,daf] = sumstats.loc[:,daf].astype("float")
+    sumstats.loc[:,eaf] = sumstats.loc[:,eaf].astype("float")
     if verbose: print(" -Plotting valriants:" + str(len(sumstats)))
     
-    
-    sumstats.loc[:,"RAF"]=sumstats[eaf] + sumstats[daf]
+    sumstats.loc[:,"RAF"]=sumstats[eaf] - sumstats[daf]
     sns.set_style("ticks")
     fig, (ax1, ax2) = plt.subplots(1, 2,**plt_args)
     ax1.scatter(sumstats["RAF"],sumstats[eaf],**scatter_args)
@@ -501,22 +525,25 @@ def plotdaf(sumstats,
         ax1.axline(xy1=(0,reg[1]),slope=reg[0],zorder=1,**reg_line_args)
     if is_threshold is True:
         if verbose: print(" -Threshold : " + str(threshold))
-        num = sum(sumstats[daf]>threshold)
+        num = sum(np.abs(sumstats[daf])>threshold )
         if verbose: print(" -Variants with relatively large DAF : ",num )
+        if verbose: print(" -Percentage for variants with relatively large DAF : ",num/len(sumstats) )
         ax1.axline(xy1=(0,threshold),slope=1,zorder=1,**threshold_line_args)
         ax1.axline(xy1=(threshold,0),slope=1,zorder=1,**threshold_line_args)
     xl,xh=ax1.get_xlim()
     yl,yh=ax1.get_ylim()
     if is_45_helper_line is True:
         ax1.axline([0,0], [1,1],zorder=1, **helper_line_args)
-    ax1.set_xlabel("Reference Alternative Allele Frequency",**fontargs)
-    ax1.set_ylabel("Effect Allele Frequency",**fontargs)
+    ax1.set_xlabel("Alternative Allele Frequency in Reference Population (RAF)",**fontargs)
+    ax1.set_ylabel("Effect Allele Frequency in Sumstats (EAF)",**fontargs)
     ax1.set_xlim([0,1])
     ax1.set_ylim([0,1])
     
     sumstats.loc[:,"ID"] = sumstats.index
-    to_plot = pd.melt(sumstats,id_vars=['ID'], value_vars=['EAF',"RAF"], var_name='Types', value_name='Allele frequency')
-    sns.kdeplot(data=to_plot, x="Allele frequency", hue="Types", fill=True, ax=ax2)
-    ax2.set_xlabel("Reference Alternative Allele Frequency",**fontargs)
+    to_plot = pd.melt(sumstats,id_vars=['ID'], value_vars=['EAF',"RAF"], var_name='Types', value_name='Allele Frequency')
+    
+    sns.histplot(data=to_plot, x="Allele Frequency", hue="Types", fill=True, ax=ax2,**histplot_args)
+    ax2.set_xlabel("Allele Frequency",**fontargs)
+    plt.tight_layout()
     return fig
     
