@@ -14,6 +14,7 @@ def mqqplot(insumstats,
           p=None,
           snpid=None,
           eaf=None,
+          mlog10p="MLOG10P",
           scaled=False,
           mode="mqq",
           mqqratio=3,
@@ -90,6 +91,8 @@ def mqqplot(insumstats,
     if p in insumstats.columns:
         # p value is necessary for all modes.
             usecols.append(p)
+    elif scaled is True:
+            usecols.append(mlog10p)
     else:
         raise ValueError("Please make sure "+p+" column is in input sumstats.")
     
@@ -136,7 +139,10 @@ def mqqplot(insumstats,
         sumstats["Annotation"]=sumstats.loc[:,anno].astype("string")   
     
     ## P value
-    sumstats["raw_P"] = pd.to_numeric(sumstats[p], errors='coerce')
+    if scaled is True:
+        sumstats["raw_P"] = pd.to_numeric(sumstats[mlog10p], errors='coerce')
+    else:
+        sumstats["raw_P"] = pd.to_numeric(sumstats[p], errors='coerce')
     
     ## CHR & POS
     if "m" in mode: 
@@ -191,7 +197,8 @@ def mqqplot(insumstats,
     
     if scaled:
         if verbose:log.write(" -P values are already converted to -log10(P)!")
-        sumstats["scaled_P"] = sumstats["raw_P"]
+        sumstats["scaled_P"] = sumstats["raw_P"].copy()
+        sumstats["raw_P"] = np.power(10,-sumstats["scaled_P"])
     else:
         if verbose:log.write(" -P values are being converted to -log10(P)...") 
         bad_p_value=len(sumstats.loc[(sumstats["raw_P"]>1)|(sumstats["raw_P"]<=0),:])
@@ -449,9 +456,9 @@ def mqqplot(insumstats,
             # stratified qq plot
             for i,(lower, upper) in enumerate(maf_bins):
                 databin = sumstats.loc[(sumstats["MAF"]>lower) &( sumstats["MAF"]<=upper),["MAF","scaled_P"]]
-                databin_raw = eaf_raw[(eaf_raw>lower)&(eaf_raw<upper)]
+                databin_raw = eaf_raw[(eaf_raw>lower) & (eaf_raw<=upper)]
                 observed = databin["scaled_P"].sort_values(ascending=False)
-                expected = -np.log10(np.linspace(minit,1,len(databin_raw)))[:len(observed)]
+                expected = -np.log10(np.linspace(minit,1,max(len(databin_raw),len(databin))))[:len(observed)]
                 label ="("+str(lower)+","+str(upper) +"]"
                 ax2.scatter(expected,observed,s=8,color=maf_bin_colors[i],label=label)
                 ax2.legend(loc="best",fontsize=10,markerscale=3,frameon=False)
